/*
 * Copyright (c) "Neo4j"
 * Neo4j Sweden AB [http://neo4j.com]
 *
 * This file is part of Neo4j.
 *
 * Neo4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.neo4j.gds.config;

import java.util.ArrayList;
import java.util.List;

import static org.neo4j.gds.config.ConfigNodesValidations.nodesNotNegative;

public final class SourceNodesFactory {

    private SourceNodesFactory() {}

    public static SourceNodes parse(Object object){
        if ( object instanceof SourceNodes) {
            return (SourceNodes) object;
        }

        else if (object instanceof List<?> list && !list.isEmpty() && list.get(0) instanceof List){
            var mapsSourceNodes = new MapSourceNodes(NodeIdParser.parseToMapOfNodeIdsWithProperties(object, SourceNodes.SOURCE_NODES_KEY));
            nodesNotNegative(mapsSourceNodes.sourceNodes(), SourceNodes.SOURCE_NODES_KEY);
            return mapsSourceNodes;
        }
        var listSourceNodes = new ListSourceNodes(NodeIdParser.parseToListOfNodeIds(object, SourceNodes.SOURCE_NODES_KEY));
        nodesNotNegative(listSourceNodes.sourceNodes(), SourceNodes.SOURCE_NODES_KEY);
        return listSourceNodes;
    }

    public static SourceNodes parseAsList(Object object) {
        if ( object instanceof SourceNodes) {
            return (SourceNodes) object;
        }
        var listSourceNodes = new ListSourceNodes(NodeIdParser.parseToListOfNodeIds(object, SourceNodes.SOURCE_NODES_KEY));
        nodesNotNegative(listSourceNodes.sourceNodes(), SourceNodes.SOURCE_NODES_KEY);
        return listSourceNodes;
    }

    public static List toMapOutput(SourceNodes sourceNodes) {
        if ( sourceNodes instanceof ListSourceNodes ) {
            return new ArrayList<>(sourceNodes.sourceNodes());
        } else if ( sourceNodes instanceof MapSourceNodes ) {
            return ((MapSourceNodes) sourceNodes).map()
                .entrySet()
                .stream()
                .map(entry -> List.of(entry.getKey(), entry.getValue()))
                .toList();
        }else{
            throw  new RuntimeException("Not valid sourceNodes");
        }
    }
}
